"use strict";

exports.__esModule = true;
exports.default = void 0;

var _constants = require("./constants");

var _utils = require("./utils");

// constants
// utils
var isArray = Array.isArray;

var createComparator = function createComparator(createIsEqual) {
  // eslint-disable-next-line no-use-before-define
  var isEqual = typeof createIsEqual === 'function' ? createIsEqual(comparator) : comparator;
  /**
   * @function comparator
   *
   * @description
   * compare the value of the two objects and return true if they are equivalent in values
   *
   * @param {any} objectA the object to test against
   * @param {any} objectB the object to test
   * @param {any} [meta] an optional meta object that is passed through to all equality test calls
   * @returns {boolean} are objectA and objectB equivalent in value
   */

  function comparator(objectA, objectB, meta) {
    if ((0, _utils.sameValueZeroEqual)(objectA, objectB)) {
      return true;
    }

    var typeOfA = typeof objectA;

    if (typeOfA !== typeof objectB || typeOfA !== 'object' || !objectA || !objectB) {
      return false;
    }

    if ((0, _utils.isPlainObject)(objectA) && (0, _utils.isPlainObject)(objectB)) {
      return (0, _utils.areObjectsEqual)(objectA, objectB, isEqual, meta);
    }

    var arrayA = isArray(objectA);
    var arrayB = isArray(objectB);

    if (arrayA || arrayB) {
      return arrayA === arrayB && (0, _utils.areArraysEqual)(objectA, objectB, isEqual, meta);
    }

    var dateA = objectA instanceof Date;
    var dateB = objectB instanceof Date;

    if (dateA || dateB) {
      return dateA === dateB && (0, _utils.sameValueZeroEqual)(objectA.getTime(), objectB.getTime());
    }

    var regexpA = objectA instanceof RegExp;
    var regexpB = objectB instanceof RegExp;

    if (regexpA || regexpB) {
      return regexpA === regexpB && (0, _utils.areRegExpsEqual)(objectA, objectB);
    }

    if ((0, _utils.isPromiseLike)(objectA) || (0, _utils.isPromiseLike)(objectB)) {
      return objectA === objectB;
    }

    if (_constants.HAS_MAP_SUPPORT) {
      var mapA = objectA instanceof Map;
      var mapB = objectB instanceof Map;

      if (mapA || mapB) {
        return mapA === mapB && (0, _utils.areMapsEqual)(objectA, objectB, isEqual, meta);
      }
    }

    if (_constants.HAS_SET_SUPPORT) {
      var setA = objectA instanceof Set;
      var setB = objectB instanceof Set;

      if (setA || setB) {
        return setA === setB && (0, _utils.areSetsEqual)(objectA, objectB, isEqual, meta);
      }
    }

    return (0, _utils.areObjectsEqual)(objectA, objectB, isEqual, meta);
  }

  return comparator;
};

var _default = createComparator;
exports.default = _default;